#!/usr/bin/python
""" NAME
        mpod.py

    SYNOPSIS
        python module

    DESCRIPTION
        Module to manage the minipods parameters/registers.

    OPTIONS
        > ...

    EXAMPLE
        > ...

    AUTHOR
        PY Duval -- 2015

    lICENSE
    (c) Copyright CERN for the benefit of the LHCb Collaboration.
    Distributed under the terms of the GNU General Public Licence V3.
    ana.py script is free and open source software.

    CHANGELOG
        PY Duval 10/02/2017 add the board device number
"""
from ctypes import c_int, byref, c_char_p, c_float, CDLL, POINTER
import sys
import getopt

# better to define LD_LIBRARY_PATH to point to the right libs directory
# ===================================================
# use a GLOBAL for library loading and init
# ===================================================
print "loading ", __name__
#lli = CDLL("/home/upgrade/PCIe40_LLI/amc40lli/lli/lib/lli-dyn.so")
lli = CDLL("lli-dyn.so")

init_done = [False]

def init(dev):
    """Init the bus parameters with default frequencies
    """
#    lli.mpod_init(125000000, 100000)
    lli.mpod_init(dev, 100000000, 100000)
    lli.mpod_setMode(1)
    lli.mpod_setTextMode(1)

#PYD new to set a different clock
# This is the first operation to do if the user doesn't want the default values
def init_clock(dev, in_clk, scl_freq):
    """Init the bus parameters
    """
    lli.mpod_init(dev, in_clk, scl_freq)
    lli.mpod_setMode(1)
    lli.mpod_setTextMode(1)

def get_init(dev):
    """ Get the bus paramaters.
    """
    cval1 = c_int()
    cval2 = c_int()
    status = lli.mpod_getInit(dev, byref(cval1), byref(cval2))
    return (status, cval1.value & 0xffffffff, cval2.value & 0xffffffff)


def read(dev, mpid, reg):
    """
    Reads the register number reg on minipod number mpid
    dev  : the board id
    mpid : the target minipod
    reg : number of the register in the device

    returns: list of two values
        status (0 success, -1 error)
        read value

    NOTE: the mask (& 0xffffffff) is necessary to have th full
    unsigned value and not a negative one with high numbers.
    """
    print "read mpod ", mpid, " register ", hex(reg)
    global init_done
    if init_done is False:
        init(dev)
    cval = c_int()
    status = lli.mpod_read(dev, mpid, reg, byref(cval))
    if (cval.value & 0xffffffff) == 0xffffffff:
        print "ERROR: read 0xffffffff at "+hex(cval.value & 0xffffffff)
    else:
        #print "OK passed read  at 0x%08x value 0x%08x" % \
        #         ((add & 0xffffffff),(cval.value & 0xffffffff)) MJ 040615
        print "OK passed read  at reg 0x%x value 0x%08x" % \
              ((reg), (cval.value & 0xffffffff))
    return (status, cval.value & 0xffffffff)


def write(dev, mpid, reg, val):
    """
    Write val on bus at address add in the register number reg
    mpid : the target minipod
    reg : number of the register in the device
    val : value to write

    returns: list of two values
        status (0 if success, -1 error)
        written value

    NOTE: the mask (& 0xffffffff) is necessary to have th full
    unsigned value and not a negative one with high numbers
    """
    print "write ", hex(val & 0xffffffff), \
          " on minipod number ", mpid, \
          " in register ", hex(reg)
    global init_done
    if init_done is False:
        init(dev)
    cval = c_int(val)
    status = lli.mpod_write(dev, mpid, reg, byref(cval))
    print "OK passed write  at reg 0x%x value 0x%08x" % \
          (reg, (cval.value & 0xffffffff))
    return (status, cval.value & 0xffffffff)


def read_bloc(dev, mpid, reg, nb_reg, data):
    """
    Reads a bloc of nb contiguous registers starting at register nb
    in minipod mpid
    dev  : the board id
    mpid : the target minipod
    reg : number of the first register to start of
    nb_reg : the number of contiguous register to read
    data : returned list of read values

    returns: status (0 success, -1 error)

    NOTE: the mask (& 0xffffffff) is necessary to have th full
    unsigned value and not a negative one with high numbers.
    """
    print "read_bloc ", nb_reg, " registers at mpod number ", mpid, \
          " starting at reg ", hex(reg)
    global init_done
    if init_done is False:
        init(dev)

    bloc = POINTER(c_int)()
    size = c_int(nb_reg)
    mpod = c_int(mpid)
    index = c_int(reg)

    status = lli.mpod_readBloc(dev, mpod, index, size, byref(bloc))

    for i in range(size.value):
#        print "read bloc 0x%x" % (bloc[i] & 0xffffffff)
        data.append(bloc[i])

    lli.ecs_free(bloc)
    return status


def get_vendor_name(dev, mpid, name):
    """
    Reads the raw (with trailing spaces) vendor name in minipod mpid
    dev  : the board id
    mpid : the target minipod
    name : returned list of a single read values

    returns: status (0 success, -1 error)
    """
    print "read vendor name from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_char_p()
    mpod = c_int(mpid)
    status = lli.mpod_getVendorName(dev, mpod, byref(data))
#    print "Read name is ", ">"+data.value+"<"
    name.append(data.value)
    lli.ecs_free(data)
    return status

def get_vendor_data(dev, mpid, data_str):
    """
    Returns a dictionary with the vendor data
    dev  : the board id
    mpid : the target minipod
    data : returned dictionary of data description

    returns: status (0 success, -1 error)
    """
    print "read vendor data from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_char_p()
    mpod = c_int(mpid)
    status = lli.mpod_getVendorData(dev, mpod, byref(data))
# split the comma separated string into several blocks
    data_list = data.value.split(",")
    for item in data_list:
        info = item.split(":")
        data_str[info[0].rstrip()] = info[1].rstrip()
    lli.ecs_free(data)
    return status

def dump_vendor_data(dev, mpid):
    """just print the values on stdout"""
    lli.mpod_dumpVendorData(dev, mpid)

def internal_temp(dev, mpid, temp):
    """ Read temperature from a minipod.
    """
    print "read temperature from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_int()
    mpod = c_int(mpid)
    status = lli.mpod_internalTemp(dev, mpod, byref(data))
    temp.append(data.value)
    return status

def internal_33_vcc(dev, mpid, volt):
    print "read 3.3 voltage from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_int()
    mpod = c_int(mpid)
    status = lli.mpod_internal33Vcc(dev, mpod, byref(data))
    volt.append(float(data.value)/10000)
    return status

def internal_25_vcc(dev, mpid, volt):
    print "read 2.5 voltage from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_int()
    mpod = c_int(mpid)
    status = lli.mpod_internal25Vcc(dev, mpod, byref(data))
    volt.append(float(data.value)/10000)
    return status

def get_light_output(dev, mpid, got_data, verbose=False):
    """Get the measures of the light output power on the 12 channels
       of a minipod.
       Returns the 12 float values in the passed array got_data
    """
    print "Get the light output power from "+str(mpid)+" in dBm"
    global init_done
    if init_done is False:
        init(dev)
    board = c_int(dev)
    mpod = c_int(mpid)
    data = (c_float*12)()
    status = lli.mpod_getLightOutput(board, mpod, byref(data))
    for i in range(12):
        if data[i] <= -float('inf'):
            #print "No power"
            got_data.append("NONE")
        else:
            got_data.append(data[i])
        if verbose is True:
            print "["+str(i)+"] ---> "+str(data[i])
    return status

def get_light_input(dev, mpid, got_data, verbose=False):
    """Get the measures of the light input power on the 12 channels
       of a minipod.
       Returns the 12 float values in the passed array got_data
    """
    print "Get the light input power into "+str(mpid)+" in dBm"
    global init_done
    if init_done is False:
        init(dev)
    board = c_int(dev)
    mpod = c_int(mpid)
    data = (c_float*12)()
    status = lli.mpod_getLightInput(board, mpod, byref(data))
    for i in range(12):
        if data[i] <= -float('inf'):
            #print "No power"
            got_data.append("NONE")
        else:
            got_data.append(data[i])
        if verbose is True:
            print "["+str(i)+"] ---> "+str(data[i])
    return status

def diff_light_rxtx(dev, mpid_tx, mpid_rx, got_data):
    """Measures the difference between the light emitted and received power
    on the 12 channels of two connected TX and RX minipods.
       Meant to check that the minipod optical parts lasers/links/diodes are OK.
       Returns the 12 float values in the passed array got_data
    """
    if mpid_tx not in [1, 3, 5, 7]:
        print "ERROR mpod "+mpid_TX+" is not emitter"
        return
    if mpid_rx not in [0, 2, 4, 6]:
        print "ERROR mpod "+mpid_RX+" is not receiver"
        return
    emt_level = []
    get_light_output(dev, mpid_tx, emt_level)
    rcv_level = []
    res = get_light_input(dev, mpid_rx, rcv_level)
    for i in range(12):
        if  emt_level[i] == "NONE":
            #print "no power on TX channel "+str(i)
            got_data.append(0.0)
            continue
        if rcv_level[i] == "NONE":
            #print "no power on RX channel "+str(i)
            got_data.append(0.0)
            continue
        got_data.append(rcv_level[i] - emt_level[i])

def dump_err_status(dev, mpid):
    """
    prints on output the values MPOD,FAULT,LOS,INT,DATA NOT READY
    """
    lli.mpod_dumpErrStatus(dev, mpid)

def get_err_status(dev, mpid, data_str):
    """
    Returns a dictionary with the error status data
    dev  : the board id
    mpid : the target minipod
    data : returned string of data description

    returns: status (0 success, -1 error)
    """
    print "read error status from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_char_p()
    mpod = c_int(mpid)
    status = lli.mpod_getErrStatus(dev, mpod, byref(data))
# split the comma separated string into several blocks
    print data.value
    data_list = data.value.split(",")
    for item in data_list:
        info = item.split(":")
        data_str[info[0].rstrip()] = info[1].rstrip()
    lli.ecs_free(data)
    return status

def get_los_status(dev, mpid, data_list):
    """
    Returns a list with the channel loss status
    dev      : the board id
    mpid     : the target minipod
    data_list : returned list of the channel loss status from 0 to 11

    returns: status (0 success, -1 error)
    """

    print "get loss status from mpod number ", mpid
    global init_done
    if init_done is False:
        init(dev)

    data = c_char_p()
    mpod = c_int(mpid)
    status = lli.mpod_losListAll(dev, mpod, byref(data))
    print "query done"
# split the comma separated string into several blocks
    data_val = data.value.split(",")
    print data_val
    for item in data_val:
        data_list.append(item)
    lli.ecs_free(data)
    return status

def main():
    """ Program to test the module.
    """
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help"])
    except getopt.error, msg:
        print msg
        print "for help use --help"
        sys.exit(2)
    for o, a in opts:
        if o in ("-h", "--help"):
            print "help"
            sys.exit(0)
    for arg in args:
        print arg  # process() is defined elsewhere

    init(0)
# should read 0x41 65
    print read(0, 1, 171)
# find a proper address   print write(1, 0x0, 0x33)

    status, val1, val2 = get_init(0)
    print "get_init() returns sysClock "+str(val1)+" SCL clock "+str(val2)
    # should return 0x41 A beginning of AVAGO
    status, val = read(0, 1, 152)
    print chr(val)

    data = []
    read_bloc(0, 1, 152, 5, data)
    for i in range(5):
        print chr(data[i])

    print "\nGetting vendor name"
    name = []
    get_vendor_name(0, 1, name)
    print "Got vendor name is ", name

    print "\nDumping vendor data"
    dump_vendor_data(0, 1)

    print "\nGetting vendor data"
    data_str = {}
    get_vendor_data(0, 1, data_str)
    print "Got data name is ", data_str
    for item in data_str:
        print item, data_str[item]

    print "\nGetting minipod temperature"
    temp = []
    internal_temp(0, 1, temp)
    print "Got temperature is ", temp

    print "\nGetting 3.3 voltage"
    volt = []
    internal_33_vcc(0, 1, volt)
    print "Got voltage is ", volt

    print "\nGetting 2.5 voltage"
    volt = []
    internal_25_vcc(0, 1, volt)
    print "Got voltage is ", volt

    print "\nDumping error status"
    dump_err_status(0, 1)

    print "\nGetting error data"
    data_str = {}
    get_err_status(0, 1, data_str)
    print "Got error status is ", data_str
    for item in data_str:
        print item

    print "\nGetting loss status"
    data_list = []
    get_los_status(0, 1, data_list)
    print "Got loss status list is ", data_list

    print "\nGetting light output power"
    data = []
    get_light_output(0, 1, data, verbose=True)

    print "\nGetting light input power"
    data = []
    get_light_input(0, 0, data, verbose=True)

    print "\nGetting light output power"
    data = []
    get_light_output(0, 3, data, verbose=True)

    print "\nGetting light input power"
    data = []
    get_light_input(0, 2, data, verbose=True)

    print "==================================================="
    print "TEST CONNECTIVITY BETWEEN 12 RX/TX pairs of channel"
    print "==================================================="
    data = []
    diff_light_rxtx(0, 1, 0, data)
    print ""
    print "Checking links from 1-->0"
    for i in range(12):
        print "["+str(i)+"] ---> "+str(data[i])

    data = []
    diff_light_rxtx(0, 3, 2, data)
    print ""
    print "Checking links from 3-->2"
    for i in range(12):
        print "["+str(i)+"] ---> "+str(data[i])

if __name__ == "__main__":
    main()
